@@ -16,13 +16,18 @@ from aiohttp.web_exceptions import (
 import asyncio
 import nest_asyncio
 
-nest_asyncio.apply()
 from typing import Optional, List, Union, Tuple, Dict, Callable
 from itertools import combinations
 from functools import reduce
 import logging
 import queue
 
+try:
+    nest_asyncio.apply()
+except RuntimeError:
+    loop = asyncio.new_event_loop()
+    asyncio.set_event_loop(loop)
+
 logger = logging.getLogger(__name__)
 
 aiohttp_errors = (
@@ -182,6 +187,7 @@ class CompoundResolver:
         agreement: Optional[int] = 1,
         batch_size: Optional[int] = None,
         n_retries: Optional[int] = 3,
+        **kwargs,
     ) -> List[Tuple[Compound, Union[List[CompoundIdentifier], None]]]:
         """Resolve a list of compound identifiers to another identifier type(s).
 
@@ -245,6 +251,7 @@ class CompoundResolver:
                 agreement=agreement,
                 batch_size=batch_size,
                 n_retries=n_retries,
+                **kwargs,
             )
         )
 
@@ -256,9 +263,14 @@ class CompoundResolver:
         agreement: Optional[int] = 1,
         batch_size: Optional[int] = None,
         n_retries: Optional[int] = 3,
+        **kwargs,
     ) -> List[Tuple[Compound, Union[List[CompoundIdentifier], None]]]:
         """This is the async function with the same API as resolve"""
 
+        # Run setup for services
+        for service in self._services:
+            await service.setup()
+
         n_identifiers = len(input_compounds)
         if batch_size is None:
             batch_size = 10 if n_identifiers >= 10 else n_identifiers
@@ -268,8 +280,15 @@ class CompoundResolver:
         backup_identifier_types = (
             backup_identifier_types if backup_identifier_types is not None else []
         )
+        progress_bar_type = kwargs.get("progress_bar_type", "tqdm")
+        if progress_bar_type == "tqdm":
+            progress_bar = tqdm
+        elif progress_bar_type == "streamlit":
+            from stqdm import stqdm
+
+            progress_bar = stqdm
         # Iterate through batches
-        for batch in tqdm(range(n_batches), position=0, desc="Batch"):
+        for batch in progress_bar(range(n_batches), position=0, desc="Batch"):
             # Get subset of data
             start = batch * batch_size
             batch_identifiers = input_compounds[start : start + batch_size]
@@ -288,7 +307,7 @@ class CompoundResolver:
                     )
                     for compound_identifier in batch_identifiers
                 ]
-                batch_bar = tqdm(
+                batch_bar = progress_bar(
                     asyncio.as_completed(tasks),
                     total=len(tasks),
                     desc=f"Batch {batch} Progress",
@@ -298,6 +317,9 @@ class CompoundResolver:
                 resolved_identifiers.extend([await f for f in batch_bar])
                 batch_bar.clear()
 
+        for service in self._services:
+            await service.teardown()
+
         return resolved_identifiers
 
     async def _resolve_one_compound(
@@ -313,7 +335,6 @@ class CompoundResolver:
         resolved_identifiers_list = []
         agreement_satisfied = False
         # Create input identifier queue
-        input_identifiers_queue = queue.Queue()
         input_identifiers_list = [
             (identifier, None) for identifier in input_compound.identifiers
         ]
@@ -332,7 +353,7 @@ class CompoundResolver:
                         output_identifier_types = [
                             output_identifier_type
                         ] + backup_identifier_types
-                        if input_identifier.identifier_type in output_identifier_types:
+                        if input_identifier.identifier_type in backup_identifier_types:
                             output_identifier_types.remove(
                                 input_identifier.identifier_type
                             )
@@ -505,6 +526,8 @@ def resolve_identifiers(
     """
     if services is None:
         services = [PubChem(autocomplete=True), CIR()]
+
+    # Convert to Compound objects
     compounds = [
         Compound(
             identifiers=[
@@ -513,6 +536,8 @@ def resolve_identifiers(
         )
         for name in names
     ]
+
+    # Do the actual resolving
     resolver = CompoundResolver(services=services, silent=silent)
     results = resolver.resolve(
         input_compounds=compounds,
@@ -521,6 +546,8 @@ def resolve_identifiers(
         agreement=agreement,
         batch_size=batch_size,
     )
+
+    # Return results
     return [
         (
             input_compound.identifiers[0].value,
